##%######################################################%##
#                                                          #
####                  Assigned BaMORC                   ####
#                                                          #
##%######################################################%##

######################### Loading all the data set for Assigned BaMORC ######################### 
data.list <- list.files("./source_code/BaMORC_GlobalOpt//Package_data/")
for(d in data.list){
        load(paste0("./source_code/BaMORC_GlobalOpt/Package_data/", d))
}

num_dat <- length(RefDB_data$carbonDat_rmGU)

library(parallel)
library(DEoptim)

######################### Helper Functions #########################
AaDensity <- function(chiStr, df=2){
        return(dchisq(chiStr, df=df))
}

ChiStr <- function(x, invM=inverseMatrices) {
        chiStr.v <- unlist(lapply(invM$getNames(), function(name){
                aa <- strsplit(name, "-")[[1]][1]
                ss <- strsplit(name, "-")[[1]][2]
                chiStr <- t(as.matrix(x) - as.matrix(c(CAMuTable[CAMuTable$Residue==aa, ss], CBMuTable[CBMuTable$Residue==aa, ss]))) %*% invM$getInvMatrix(name) %*% (as.matrix(x) - as.matrix(c(CAMuTable[CAMuTable$Residue==aa, ss], CBMuTable[CBMuTable$Residue==aa, ss])))
                x <- rev(x)
                chiStr_rev <- t(as.matrix(x) - as.matrix(c(CAMuTable[CAMuTable$Residue==aa, ss], CBMuTable[CBMuTable$Residue==aa, ss]))) %*% invM$getInvMatrix(name) %*% (as.matrix(x) - as.matrix(c(CAMuTable[CAMuTable$Residue==aa, ss], CBMuTable[CBMuTable$Residue==aa, ss])))
                return(min(chiStr, chiStr_rev))
        }))
        #print(chiStr.v)
        
        chiStr.table <- data.table::data.table(cbind(AaCodesSingle.v,matrix(chiStr.v, ncol = 3, nrow = 20, byrow = T)))
        colnames(chiStr.table) <- c("AA", "H", "B", "C")
        return(chiStr.table)
}

AssignedRefCor <- function(input_data, from=-5, to=5){
        #################### Calculate frequence given the assignment #################### 
        # Constructure temperary frequency table
        AA_SS <- paste(input_data$AA, input_data$SS, sep = "-")
        aaFreq <- data.frame(AA_SS, Frequency=1)
        
        fn <- function(x){
                InvDet.v <- sapply(aaFreq$AA_SS, function(aass){
                        det(inverseMatrices$getInvMatrix(aass))
                })
                est_prob <- unlist(lapply(c(1:nrow(input_data)), function(i){
                        CaCb.cs <- unlist(c(input_data[i,4] + x, input_data[i,5] + x))
                        aa <- unlist(input_data[i,2])
                        ss <- unlist(input_data[i,3])
                        invM <- inverseMatrices$getInvMatrix(paste(aa, ss, sep = "-"))
                        chiStr <- t(as.matrix(CaCb.cs) - as.matrix(c(CAMuTable[CAMuTable$Residue==aa, ss], CBMuTable[CBMuTable$Residue==aa, ss]))) %*% 
                                invM %*% 
                                (as.matrix(CaCb.cs) - as.matrix(c(CAMuTable[CAMuTable$Residue==aa, ss], CBMuTable[CBMuTable$Residue==aa, ss])))
                        den <-  AaDensity(chiStr)
                        if(aa == "C"){
                                aa <- "B"
                                ss <- unlist(input_data[i,3])
                                invM <- inverseMatrices$getInvMatrix(paste(aa, ss, sep = "-"))
                                chiStr <- t(as.matrix(CaCb.cs) - as.matrix(c(CAMuTable[CAMuTable$Residue==aa, ss], CBMuTable[CBMuTable$Residue==aa, ss]))) %*% 
                                        invM %*% 
                                        (as.matrix(CaCb.cs) - as.matrix(c(CAMuTable[CAMuTable$Residue==aa, ss], CBMuTable[CBMuTable$Residue==aa, ss])))
                                if(AaDensity(chiStr) > den) {
                                        den <- AaDensity(chiStr)
                                        InvDet.v[i] <- det(invM)
                                }
                                
                        }
                        
                        return(den)
                }))
                est_prob <- est_prob*InvDet.v
                
                sumAbsDiff <- sum((est_prob- aaFreq$Frequency)^2)
                return(sumAbsDiff)
                
        }
        
        est.ras10 <- DEoptim(fn, lower=from, upper=to, control=list(storepopfrom=1, trace=FALSE, itermax=10, CR=1))
        return(est.ras10)
}

######################### Main Functions #########################
results_ <- mclapply(c(1:num_dat), function(indx){
        out <- tryCatch(
                {
                        input_data <- RefDB_data$carbonDat_rmGU[[indx]]
                        result <- AssignedRefCor(input_data=input_data)
                        return(result)
                },
                error=function(error_condition){
                        return(NA)
                }
                
        )
        
        
        return(out)
        
}, mc.cores = 1)

save(results_, file = paste0("../output/Assigned_BaMORC/Assigned_BaMORC.RData"))