##%######################################################%##
#                                                          #
####                Covariance B                        ####
#                                                          #
##%######################################################%##

setwd("./source_code/BMR6032_Analysis/")
library(data.table)
library(dplyr)
library(tidyr)
library(parallel)

dir <-"../data/"
# Declare all the naming variables

AaCodesSingle.v = c("A","R","N","D","B","C","Q","E", "H","I","L","K","M","F","P","S","T","Y","W","V")
AaCodes3LetterAllCap.v = c("ALA", "ARG", "ASN", "ASP", "CYS", "GLN", "GLU", "HIS", "ILE", "LEU", "LYS", "MET", "PHE", "PRO", "SER", "THR", "TYR","TRP","VAL")
AaCodes3Letter1stCap.v = c("Ala", "Arg", "Asn", "Asp", "Cys", "Gln", "Glu", "His", "Ile", "Leu", "Lys", "Met", "Phe", "Pro", "Ser", "Thr", "Tyr","Trp","Val")
SecondaryStructures.v  = c("H", "B", "C")
Resonances.v = c("CA", "CB")
options(digits=16)

aa.LookUp <- c("A"="Ala","R"="ARG","N"="ASN","D"="ASP","C"="CYS","Q"="GLN","E"="GLU","H"="HIS","I"="ILE","L"="LEU","K"="LYS","M"="MET","F"="PHE","P"="PRO","S"="SER","T"="THR","Y"="TYR","W"="TRP","V"="VAL",
               "ALA"="A","ARG"="R","ASN"="N","ASP"="D","CYS"="C","GLN"="Q","GLU"="E","HIS"="H","ILE"="I","LEU"="L","LYS"="K","MET"="M","PHE"="F", "PRO"="P","SER"="S","THR"="T","TYR"="Y","TRP"="W","VAL"="V")

RefDB.StatCA <- read.csv(paste(dir, "RefDB.caStat.csv", sep=""))
RefDB.StatCB <- read.csv(paste(dir, "RefDB.cbStat.csv", sep=""))
RefDB.StatCB <- RefDB.StatCB[which(RefDB.StatCB$Residue!="Gly"),] # Remove the glycine
CarbonCov.t <- read.csv(paste(dir, "CarbonCovTable_BC.csv", sep=""))

# Reading in the population statistics
PopStats <- read.table(paste(dir, "aaPopStat.csv", sep=""))


# Generate average covariance
AvgCov.t <- read.csv(paste(dir, "avgCov_BC.csv", sep="")) # rowName is from the AaCodesSingle.v
# Generate covariance based on secondary structure
CarbonCov.t <- CarbonCov.t[-grep("-A", CarbonCov.t$"AA.SS"),]

# Import the file name of the RefDB data
FileList.v = list.files(path = paste(dir, "Generated-NA/", pattern="txt"))
FileList.noExt.v = gsub(".txt", "", FileList.v)

# Find all the mean and sd:
CASdTable = data.frame(RefDB.StatCA$Residue, RefDB.StatCA$Coil.sd, RefDB.StatCA$Helix.sd, RefDB.StatCA$Beta.sd, RefDB.StatCA$Avg.sd)
colnames(CASdTable) = c("Residue", "C", "H", "B", "A")

CBSdTable = data.frame(RefDB.StatCB$Residue, RefDB.StatCB$Coil.sd, RefDB.StatCB$Helix.sd, RefDB.StatCB$Beta.sd, RefDB.StatCB$Avg.sd)
colnames(CBSdTable) = c("Residue", "C", "H", "B", "A")

CAMuTable = data.frame(RefDB.StatCA$Residue, RefDB.StatCA$Coil.mu, RefDB.StatCA$Helix.mu, RefDB.StatCA$Beta.mu, RefDB.StatCA$Avg.mu)
colnames(CAMuTable) = c("Residue", "C", "H", "B", "A")

CBMuTable = data.frame(RefDB.StatCB$Residue, RefDB.StatCB$Coil.mu, RefDB.StatCB$Helix.mu, RefDB.StatCB$Beta.mu, RefDB.StatCB$Avg.mu)
colnames(CBMuTable) = c("Residue", "C", "H", "B", "A")

# Load all the data
load(paste0(dir, "RefDB.dat")) # load dat.refDB
load(paste0(dir, "StatCA")) # 
load(paste0(dir, "StatCB"))
load(paste0(dir, "StatCov.dat"))
load(paste0(dir, "invCovMatrix"))

TestData.f = function(filename, error=0){
        # mac:
        filename.test = paste( dir, filename, sep = "" )
        # filename.test = paste( dir, "RefDB_files/", filename, sep = "" )
        data <- read.table( filename.test, quote="\"" )
        # remove GLY
        data.no.gly = data[which(data$V3!="GLY"),]
        
        data.ca <- data.no.gly[which(data.no.gly$V4=="CA"),]
        data.cb <- data.no.gly[which(data.no.gly$V4=="CB"),]
        if(nrow(data.ca)==0){
                return(0)
        }
        common.index = intersect(data.ca$V2, data.cb$V2)
        
        ca.cs = c()
        cb.cs = c()
        resonance.name = c()
        for(i in common.index){
                ca.cs = c(ca.cs, data.ca[which(data.ca$V2 == i),]$V6)
                cb.cs = c(cb.cs, data.cb[which(data.cb$V2 == i),]$V6)
                resonance.name = c(resonance.name, toString(data.ca[which(data.ca$V2 == i),]$V3))
        }
        
        data.f = data.frame(resonance.name, ca.cs+error, cb.cs+error)
        colnames(data.f) = c("AA", "CA", "CB")
        return(data.f)
}

ChiStr.f = function(dataCaCb.v, SecStr, invCovMatList){
        #' cacb.v is the matrix of [x.ca, x.cb]
        #' secondaryStructure is one sec.str from ss
        names = c()
        chiStr.v = c()
        ss = SecStr
        for(aa in AaCodesSingle.v){
                name = paste(aa, SecStr, sep="-")
                names = c(names, name)
                # Find the mu for each aa and ss.
                caMu = as.double( CAMuTable[CAMuTable$Residue == aa,][ss] )
                cbMu = as.double( CBMuTable[CBMuTable$Residue == aa,][ss] )
                mean.m = as.matrix(c(caMu, cbMu))
                #print(mean.m)
                invCov = invCovMatList[[name]]
                #print(name)
                #print(invCov)
                dataCaCb.m = as.matrix(dataCaCb.v)
                #print(dataCaCb.m)
                chiStr = t(dataCaCb.m - mean.m) %*% invCov %*%(dataCaCb.m - mean.m)
                #print(chiStr)
                #'     if(chiStr < 0){
                #'        chiStr = 0
                #'     }
                #print(chiStr)
                chiStr.v = c(chiStr.v, chiStr)
        }
        chiStr.t = data.frame(names, chiStr.v)
        return(chiStr.t)
}

AaProb.f = function(chiStr){
        return(dchisq(chiStr, df=2))
}

AaFreq.f = function(seq){
        AaCodes.v = c("A","R","N","D","C","Q","E","H","I","L","K","M","F","P","S","T","Y","W","V")
        counts = table(seq <- strsplit(gsub("G","", seq), split = ""))
        freq.v <- sapply(AaCodes.v, function(aa){ counts[aa]/sum(counts)})
        freq.v[is.na(freq.v)] <- 0
        return(data.frame(AA = AaCodes.v, freq = freq.v, row.names = NULL))
}

SumAbsDiff.f = function(step, testData, aaFreq, invCovMatList){
        testDataLen = length(testData$"AA")
        sumAaProb.v = rep(0, 19)
        
        for(i in 1:testDataLen){
                aaProb.v = c()
                CaCbCS.v = c(testData[i,][,2]+step, testData[i,][,3]+step) # this is generate the new CA & CB Chemical shift value by adding the step
                # print(newCaCbCSValue.v)
                # pick one row with 2 elements
                # calculate the chiStr with 3 structures
                
                chiStrH.v = ChiStr.f(CaCbCS.v, SecStr = "H", invCovMatList)   # return a list of 20 chiStr.v for each
                chiStrB.v = ChiStr.f(CaCbCS.v, SecStr = "B", invCovMatList)
                chiStrC.v = ChiStr.f(CaCbCS.v, SecStr = "C", invCovMatList)
                
                # flip the Ca and CB cs.
                CaCbCS.v.flip = rev(CaCbCS.v)
                chiStrH.r.v = ChiStr.f(CaCbCS.v.flip, SecStr = "H", invCovMatList)   # return a list of 20 chiStr.v for each
                chiStrB.r.v = ChiStr.f(CaCbCS.v.flip, SecStr = "B", invCovMatList)
                chiStrC.r.v = ChiStr.f(CaCbCS.v.flip, SecStr = "C", invCovMatList)
                
                for(aaName in AaCodesSingle.v){
                        name.H = paste(aaName, "H", sep = "-")
                        name.B = paste(aaName, "B", sep = "-")
                        name.C = paste(aaName, "C", sep = "-")
                        
                        names = c(name.H, name.B, name.C)
                        
                        chiStr = min( chiStrH.v[chiStrH.v$name == name.H,][,2], 
                                      chiStrB.v[chiStrB.v$name == name.B,][,2], 
                                      chiStrC.v[chiStrC.v$name == name.C,][,2], 
                                      chiStrH.r.v[chiStrH.r.v$name == name.H,][,2], 
                                      chiStrB.r.v[chiStrB.r.v$name == name.B,][,2], 
                                      chiStrC.r.v[chiStrC.r.v$name == name.C,][,2])
                        name = names[which.min(c( chiStrH.v[chiStrH.v$name == name.H,][,2], 
                                                  chiStrB.v[chiStrB.v$name == name.B,][,2], 
                                                  chiStrC.v[chiStrC.v$name == name.C,][,2], 
                                                  chiStrH.r.v[chiStrH.r.v$name == name.H,][,2], 
                                                  chiStrB.r.v[chiStrB.r.v$name == name.B,][,2], 
                                                  chiStrC.r.v[chiStrC.r.v$name == name.C,][,2]))]
                        
                        #print(name)
                        aaProb = AaProb.f(chiStr)
                        # if(is.nan(aaProb)) {aaProb=0}
                        aaProb.v = c(aaProb.v, aaProb)
                }
                aaProb.v[5] <- sum(aaProb.v[5],aaProb.v[20])
                aaProb.v <- aaProb.v[-20]
                sumAaProb.v = sumAaProb.v + aaProb.v
        }
        #print(sumAaProb.v)
        aaPredProb.v = sumAaProb.v / sum(sumAaProb.v)
        #print(aaPredProb.v)
        sumAbsDiff = sum( abs( aaPredProb.v - as.double(aaFreq[,"freq"]) ) )
        # print(sumAbsDiff)
        return(sumAbsDiff)
}

RefCorr.f = function(filename, error, from, to, invCovMatList, sequence){
        N = seq(from, to, length.out = 50)                 # generate the steps for test
        testData = TestData.f(filename, error)
        aaFreq = AaFreq.f(sequence)
        # mean.v = c(mean(testData$CA), mean(testData$CB))
        sumAbsDiff.v = mclapply(N, function(step){SumAbsDiff.f(step, testData, aaFreq, invCovMatList)}, mc.cores = 1)
        # sumAbsDiff.t = cbind(N, sumAbsDiff.v)
        # colnames(sumAbsDiff.t) = c("steps", "sumAbsDiff")
        # print(sumAbsDiff.t)
        return(unlist(sumAbsDiff.v))
}

testing.f = function(filename, error = 0, from = -5, to = 5, invCovMatList = invCovMatrix, sequence=s){
        # print(percent)
        result1 = RefCorr.f(filename, error, from, to, invCovMatList, sequence)
        if(!is.double(result1)) return()
        N1 = seq(from, to, length.out = 50)
        #    plot(N,result1)
        lowest1 = N1[which.min(result1)]
        from = lowest1 - 1
        to = lowest1 + 1
        
        result2 = RefCorr.f(filename, error, from, to, invCovMatList, sequence)
        N2 = seq(from, to, length.out = 50)
        lowest2 = N2[which.min(result2)]
        result = list()
        result[[1]] = result1
        result[[2]] = result2
        result[[3]] = lowest1
        result[[4]] = lowest2
        result[[5]] = N1
        result[[6]] = N2
        
        # print(result)
        return(result)
}


s = "ATKIDKEACRAAYNLVRDDSAVIWVTFRYDATIVPDQADYQHFIQQCTDDVRLFAFVRFTTDAMSKRSKFALITWIEDVSLQRAKTTDKTLVKEVVQNFAKEFVISDRKELEEDFIRSELKKAANYDAQS"
results <- testing.f(filename = "bmr6032.str.corr.txt", sequence=s)

save(file = "../../output/BMR6032_Analysis/Covariance_B.RData", results)

############################################ Generating Figures
library(ggplot2)
result1 <- data.frame(Step = results[[5]], AbsSumDiff = results[[1]])
result2 <- data.frame(Step = results[[6]], AbsSumDiff = results[[2]])
lowest1 <- results[[3]]
lowest2 <- results[[4]]

science_theme = theme(axis.line = element_line(size = 0.7, color = "black"), legend.position = "none", text = element_text(size = 14, color = "black"), axis.text=element_text(size=15), axis.title=element_text(size=17), panel.background = element_rect(fill = "white"))

bill_formatter <- function(x) {
        lab <- signif(x, digits = 2) 
}

plot <- ggplot(result2, aes(Step, AbsSumDiff)) + science_theme +
        geom_point(size = 2) +
        geom_vline(xintercept = 0, col = "red", size = 2, alpha = 0.7) +
        geom_vline(xintercept = lowest2, col = "blue", size = 2, alpha = 0.7) +
        ggtitle("Method B for BMR6032 Second 50 Scans") + 
        scale_y_continuous(label=bill_formatter)

print(plot)

ggsave("../../output/BMR6032_Analysis/Covariance_B.png", width = 5, height = 4, units = "in")
