##%######################################################%##
#                                                          #
####                    Covariance D                    ####
#                                                          #
##%######################################################%##
setwd("./source_code/BMR6032_Analysis/")
dir <-"../data/"
# Declare all the naming variables

AaCodesSingle.v = c("A","R","N","D","B","C","Q","E", "H","I","L","K","M","F","P","S","T","Y","W","V")
AaCodes3LetterAllCap.v = c("ALA", "ARG", "ASN", "ASP", "CYS", "GLN", "GLU", "HIS", "ILE", "LEU", "LYS", "MET", "PHE", "PRO", "SER", "THR", "TYR","TRP","VAL")
AaCodes3Letter1stCap.v = c("Ala", "Arg", "Asn", "Asp", "Cys", "Gln", "Glu", "His", "Ile", "Leu", "Lys", "Met", "Phe", "Pro", "Ser", "Thr", "Tyr","Trp","Val")
SecondaryStructures.v  = c("H", "B", "C", "A")
Resonances.v = c("CA", "CB")
options(digits=16)

# Reading in all the statisitcs
RefDB.StatCA <- read.csv(paste(dir, "RefDB.caStat.csv", sep=""))
RefDB.StatCB <- read.csv(paste(dir, "RefDB.cbStat.csv", sep=""))
RefDB.StatCB <- RefDB.StatCB[which(RefDB.StatCB$Residue!="Gly"),] # Remove the glycine
CarbonCov.t <- read.csv(paste(dir, "CarbonCovTable_BC.csv", sep=""))

# Reading in the population statistics
PopStats <- read.table(paste(dir, "aaPopStat.csv", sep=""))


# Generate average covariance
AvgCov.t <- read.csv(paste(dir, "avgCov_BC.csv", sep="")) # rowName is from the AaCodesSingle.v
# Generate covariance based on secondary structure
CarbonCov.t <- CarbonCov.t[-grep("-A", CarbonCov.t$"AA.SS"),]

# Import the file name of the RefDB data
FileList.v = list.files(path = paste(dir, "Generated-NA/", pattern="txt"))
FileList.noExt.v = gsub(".txt", "", FileList.v)

# Find all the mean and sd:
CASdTable = data.frame(RefDB.StatCA$Residue, RefDB.StatCA$Coil.sd, RefDB.StatCA$Helix.sd, RefDB.StatCA$Beta.sd, RefDB.StatCA$Avg.sd)
colnames(CASdTable) = c("Residue", "C", "H", "B", "A")

CBSdTable = data.frame(RefDB.StatCB$Residue, RefDB.StatCB$Coil.sd, RefDB.StatCB$Helix.sd, RefDB.StatCB$Beta.sd, RefDB.StatCB$Avg.sd)
colnames(CBSdTable) = c("Residue", "C", "H", "B", "A")

CAMuTable = data.frame(RefDB.StatCA$Residue, RefDB.StatCA$Coil.mu, RefDB.StatCA$Helix.mu, RefDB.StatCA$Beta.mu, RefDB.StatCA$Avg.mu)
colnames(CAMuTable) = c("Residue", "C", "H", "B", "A")

CBMuTable = data.frame(RefDB.StatCB$Residue, RefDB.StatCB$Coil.mu, RefDB.StatCB$Helix.mu, RefDB.StatCB$Beta.mu, RefDB.StatCB$Avg.mu)
colnames(CBMuTable) = c("Residue", "C", "H", "B", "A")

TestData.f = function(filename, error=0){
        # mac:
        filename.test = paste( dir, filename, sep = "" )
        # filename.test = paste( dir, "RefDB_files/", filename, sep = "" )
        data <- read.table( filename.test, quote="\"" )
        # remove GLY
        data.no.gly = data[which(data$V3!="GLY"),]
        
        data.ca <- data.no.gly[which(data.no.gly$V4=="CA"),]
        data.cb <- data.no.gly[which(data.no.gly$V4=="CB"),]
        if(nrow(data.ca)==0){
                return(0)
        }
        common.index = intersect(data.ca$V2, data.cb$V2)
        
        ca.cs = c()
        cb.cs = c()
        resonance.name = c()
        for(i in common.index){
                ca.cs = c(ca.cs, data.ca[which(data.ca$V2 == i),]$V6)
                cb.cs = c(cb.cs, data.cb[which(data.cb$V2 == i),]$V6)
                resonance.name = c(resonance.name, toString(data.ca[which(data.ca$V2 == i),]$V3))
        }
        
        data.f = data.frame(resonance.name, ca.cs+error, cb.cs+error)
        colnames(data.f) = c("AA", "CA", "CB")
        return(data.f)
}

covMatrix.C.f <- function( aaName){
        sd.ca = PopStats[aaName,"CA_Sd"]
        sd.cb = PopStats[aaName,"CB_Sd"]
        sd.ca.n = as.double(sd.ca)
        sd.cb.n = as.double(sd.cb)
        
        v <- PopStats[aaName,"Cov"] # -> there's no B or C residue only C
        covMatrix = matrix(c(sd.ca.n^2, v, v, sd.cb.n^2), 2 , 2, byrow = T)
        return(covMatrix)
}
# covMatrix.C.f( "A")
# 
invCovMatList.l.C = list()
for(aa in row.names(PopStats) ){
        CovMatrix = covMatrix.C.f(aa)
        invCovMatrix = solve(CovMatrix)
        invCovMatList.l.C[[aa]] = invCovMatrix
}

ChiStr.C.f = function(invCovMatList, dataCaCb.v){
        #' dataCaCb.v is one row of Ca & Cb from data
        names = c()
        chiStr.v = c()
        for(aa in row.names(PopStats)){
                names = c(names, aa)
                # Find the mu for each aa and ss.
                caMu = as.double( PopStats[aa, "CA_Mu"] )
                cbMu = as.double( PopStats[aa, "CB_Mu"] )
                mean.m = as.matrix(c(caMu, cbMu))
                #print(mean.m)
                invCov = invCovMatList[[aa]]
                #print(name)
                #print(invCov)
                dataCaCb = as.matrix(dataCaCb.v)
                #print(dataCaCb.m)
                chiStr = t(dataCaCb - mean.m) %*% invCov %*%(dataCaCb - mean.m)
                chiStr.v = c(chiStr.v, chiStr)
                #print(chiStr.v)
        }
        chiStr.t = data.frame(names, chiStr.v)
        chiStr.t <-na.omit(chiStr.t)
        # print(chiStr.t)
        return(chiStr.t)
}

knownProb.f <- function(seq){
        freq <- as.data.frame(table(seq))
        names(freq) <- c("AA", "Freq")
        freq$Freq[is.na(freq$Freq)] <- 0
        known.prob <- freq$Freq/sum(freq$Freq)
        names(known.prob) <- freq$AA
        return(known.prob)
}

AaProb.f = function(chiStr){
        prob = dchisq(chiStr, df=2)
        return(prob)
}

AaFreq.f = function(seq){
        AaCodesSingle.v = c("A","R","N","D","C","Q","E","H","I","L","K","M","F","P","S","T","Y","W","V")
        seq <- gsub("G","", seq)
        n <- nchar(seq)
        seq <- as.data.frame(strsplit(seq, split = "") )
        freq.v = c()
        for(aa in AaCodesSingle.v){ # here the has 19 amino acid names
                AA.freq = sum(seq==aa)/n
                freq.v = c(freq.v, AA.freq)
        }
        freqTable.t = cbind(AaCodesSingle.v, freq.v)
        colnames(freqTable.t) = c("AA", "freq")
        return(freqTable.t)
}

SumAbsDiff.f = function(step, testData, aaFreq, invCovMatList = invCovMatList.l.C){
        testDataLen = nrow(testData)
        sumAaProb.v = rep(0, 19)
        
        for(i in 1:testDataLen){
                aaProb.v = c()
                
                CaCbCS.v = c(testData[i,2]+step, testData[i,3]+step) 
                CaCbCS.v.flip = rev(CaCbCS.v)
                
                chiStrs.l.v      = ChiStr.C.f(invCovMatList, CaCbCS.v)
                chiStrs.l.v.flip = ChiStr.C.f(invCovMatList, CaCbCS.v.flip)
                
                aaProb.l = sapply(names(invCovMatList), function(aa){
                        chiStr = chiStrs.l.v[which(chiStrs.l.v$names==aa),2]
                        chiStr.flip = chiStrs.l.v.flip[which(chiStrs.l.v.flip$names==aa),2]
                        minValue <- min(chiStr, chiStr.flip)
                        return(AaProb.f(minValue ) )
                })
                sumAaProb.v = sumAaProb.v + aaProb.l
        }
        #print(sumAaProb.v)
        aaPredProb.v = sumAaProb.v / sum(sumAaProb.v)
        #print(aaPredProb.v)
        sumAbsDiff = sum( abs( aaPredProb.v - as.double(aaFreq[,2]) ) )
        #print(sumAbsDiff)
        return(sumAbsDiff)
}

RefCorr.f = function(filename, error, from, to, sequence){
        N <- seq(from, to, length.out = 50)                 # generate the steps for test
        # print(testData)
        testData = TestData.f(filename)
        aaFreq = AaFreq.f(sequence)
        # mean.v = c(mean(testData$CA), mean(testData$CB))
        sumAbsDiff.v = c()
        for(step in N){
                #print(step)
                sumAbsDiff.v = c(sumAbsDiff.v, SumAbsDiff.f(step, testData, aaFreq))
        }
        
        sumAbsDiff.t = cbind(N, sumAbsDiff.v)
        colnames(sumAbsDiff.t) = c("steps", "sumAbsDiff")
        # print(range(sumAbsDiff.t))
        # return( sumAbsDiff.t[ which.min(sumAbsDiff.t[,"sumAbsDiff"]), ] )
        # print(sumAbsDiff.v)
        return(sumAbsDiff.v)
}

testing.f = function(filename, error = 0, from = -5, to = 5, sequence=s){
        # print(percent)
        result1 = RefCorr.f(filename, error = 0, from = -5, to = 5, sequence = s)
        if (!is.double(result1)) return()
        N1 = seq(from, to, length.out = 50)
        # plot(N,result1, type = "b", main = "Method A", xlab = "Steps")
        lowest1 = N1[which.min(result1)]
        from = lowest1 - 1
        to = lowest1 + 1
        
        result2 = RefCorr.f(filename, error = 0, from, to, sequence = s)
        N2 = seq(from, to, length.out = 50)
        # plot(N,result2)
        lowest2 = N2[which.min(result2)]
        result = list()
        result[[1]] = result1
        result[[2]] = result2
        result[[3]] = lowest1
        result[[4]] = lowest2
        result[[5]] = N1
        result[[6]] = N2
        
        return(result)
}

s ="ATKIDKEACRAAYNLVRDDSAVIWVTFRYDATIVPDQADYQHFIQQCTDDVRLFAFVRFTTDAMSKRSKFALITWIEDVSLQRAKTTDKTLVKEVVQNFAKEFVISDRKELEEDFIRSELKKAANYDAQS"
results <- testing.f(filename = "bmr6032.str.corr.txt", sequence = s)

save(file = "../../output/BMR6032_Analysis/Covariance_D.RData", results)

############################################ Generating Figures
library(ggplot2)
result1 <- data.frame(Step = results[[5]], AbsSumDiff = results[[1]])
result2 <- data.frame(Step = results[[6]], AbsSumDiff = results[[2]])
lowest1 <- results[[3]]
lowest2 <- results[[4]]

science_theme = theme(axis.line = element_line(size = 0.7, color = "black"), legend.position = "none", text = element_text(size = 14, color = "black"), axis.text=element_text(size=15), axis.title=element_text(size=17), panel.background = element_rect(fill = "white"))

bill_formatter <- function(x) {
        lab <- signif(x, digits = 2) 
}

plot <- ggplot(result2, aes(Step, AbsSumDiff)) + science_theme +
        geom_point(size = 2) +
        geom_vline(xintercept = 0, col = "red", size = 2, alpha = 0.7) +
        geom_vline(xintercept = lowest2, col = "blue", size = 2, alpha = 0.7) +
        ggtitle("Method D for BMR6032 Second 50 Scans") + 
        scale_y_continuous(label=bill_formatter)

print(plot)

ggsave("../../output/BMR6032_Analysis/Covariance_D.png", width = 5, height = 4, units = "in")
