##%######################################################%##
#                                                          #
####                  Assigned BaMORC                   ####
#                                                          #
##%######################################################%##

######################### Loading all the data set for BaMORC_GO ######################### 
data.list <- list.files("./source_code/Assigned_BaMORC/Package_data/")
for(d in data.list){
        load(paste0("./source_code/Assigned_BaMORC/Package_data/", d))
}

num_dat <- length(RefDB_data$carbonDat_rmGU)

library(parallel)
library(DEoptim)

######################### Helper Functions ######################### 
AaDensity <- function(chiStr, df=2){
        return(dchisq(chiStr, df=df))
}

ChiStr <- function(x, invM=inverseMatrices) {
        chiStr.v <- unlist(lapply(invM$getNames(), function(name){
                aa <- strsplit(name, "-")[[1]][1]
                ss <- strsplit(name, "-")[[1]][2]
                chiStr <- t(as.matrix(x) - as.matrix(c(CAMuTable[CAMuTable$Residue==aa, ss], CBMuTable[CBMuTable$Residue==aa, ss]))) %*% invM$getInvMatrix(name) %*% (as.matrix(x) - as.matrix(c(CAMuTable[CAMuTable$Residue==aa, ss], CBMuTable[CBMuTable$Residue==aa, ss])))
                x <- rev(x)
                chiStr_rev <- t(as.matrix(x) - as.matrix(c(CAMuTable[CAMuTable$Residue==aa, ss], CBMuTable[CBMuTable$Residue==aa, ss]))) %*% invM$getInvMatrix(name) %*% (as.matrix(x) - as.matrix(c(CAMuTable[CAMuTable$Residue==aa, ss], CBMuTable[CBMuTable$Residue==aa, ss])))
                return(min(chiStr, chiStr_rev))
        }))
        #print(chiStr.v)
        
        chiStr.table <- data.table::data.table(cbind(AaCodesSingle.v,matrix(chiStr.v, ncol = 3, nrow = 20, byrow = T)))
        colnames(chiStr.table) <- c("AA", "H", "B", "C")
        return(chiStr.table)
}

SumSquredDiff <- function(x, dat_cacb, aaFreq) {
        
        sumAaProb.v <- apply(
                do.call(cbind, lapply(c(1:nrow(dat_cacb)), function(i){
                        CaCbCS.v <- unlist(c(dat_cacb[i,1] + x, dat_cacb[i,2] + x))
                        chiStar.v <- as.numeric(as.vector(t(ChiStr(CaCbCS.v)[,2:4])))
                        
                        density.v <- AaDensity(chiStar.v)
                        names(density.v) <- cname
                        C_index <- which(cname == c("C-H","C-B","C-C"))
                        B_index <- which(cname == c("B-H","B-B","B-C"))
                        density.v[C_index] <- density.v[C_index] + density.v[B_index]
                        density.v <- density.v[-B_index]
                        return(density.v)
                })),
                1,
                sum)*AA57OLWeights
        
        sumAaProb.v <- sumAaProb.v/sum(sumAaProb.v)
        
        actualFreq.v <- aaFreq[[2]]
        actualFreq.v <- actualFreq.v %*% as.matrix(AA57OLMatrix)
        actualFreq.v <- actualFreq.v*AA57OLWeights
        actualFreq.v <- actualFreq.v/sum(actualFreq.v)
        
        SumSquredDiff <- sum((sumAaProb.v- actualFreq.v)^2)/length(sumAaProb.v)
        
        return(SumSquredDiff)
}

SinglePeptideDeOptim = function(input_data, from=-5, to=5, aaFreq, control=10){
        input_data <- input_data[,c(4:5)]
        est.ras50 <- DEoptim(SumSquredDiff, lower=from, upper=to, dat_cacb=input_data, aaFreq=aaFreq, control=list(storepopfrom=1, trace=FALSE, itermax=control, CR=1))
        return(est.ras50)
}

######################### Main Function ######################### 
results_ <- mclapply(c(1:num_dat), function(indx){
        out <- tryCatch(
                {
                        input_data <- RefDB_data$carbonDat_rmGU[[indx]]
                        freq <- RefDB_data$getFreq(indx)
                        result <- SinglePeptideDeOptim(input_data=input_data, from=-5, to=5, aaFreq=freq)
                        return(result)
                },
                error=function(error_condition){
                        return(NA)
                }
                
        )
        
        
        return(out)
        
}, mc.cores = 1)

save(results_, file = paste0("../output/BaMORC_GO/BaMORC_GO.RData"))