
##%######################################################%##
#                                                          #
####                 Cysteine Analysis                  ####
#                                                          #
##%######################################################%##
setwd("./source_code/Cysteine_Analysis/")
library(ggExtra)
library(ggplot2)
require(cluster)
require(class)

dat.dir <- "../data/Generated-NA/"
filenames <- list.files(dat.dir)

# remove U and glycine
filenames <- filenames[!grepl("U|G", filenames)]


filenames.CA <- filenames[grepl("CA", filenames)]
filenames.CB <- filenames[grepl("CB", filenames)]
filename.t <- data.frame(CA=filenames.CA, CB=filenames.CB, stringsAsFactors = F)

ala.filenames <- filename.t[1:3,]
cys.filenames <- filename.t[4:6,]

remove_outliers <- function(x, na.rm = TRUE, ...) {
        qnt <- quantile(x, probs=c(.25, .75), na.rm = na.rm, ...)
        H <- 3 * IQR(x, na.rm = na.rm)
        y <- x
        y[x < (qnt[1] - H)] <- NA
        y[x > (qnt[2] + H)] <- NA
        return(y)
}

corr_eqn <- function(x,y, digits = 2) {
        corr_coef <- round(cor(x, y), digits = digits)
        paste("r = ", corr_coef)
}

SS <- c("Beta_Strand", "Coil", "Helix")

ala_names <- c("Ala_betaStrand", "Ala_Coil", "Ala_Helix")
for(i in 1:nrow(ala.filenames)){
        # t <- paste("Alanine", SS[i], sep=" ")
        CS.CA <- read.table(paste0(dat.dir, ala.filenames[i,1]), quote="\"", comment.char="")
        CS.CA <- remove_outliers(unlist(CS.CA))
        CS.CB <- read.table(paste0(dat.dir, ala.filenames[i,2]), quote="\"", comment.char="")
        CS.CB <- remove_outliers(unlist(CS.CB))
        dat <- data.frame(CA = CS.CA, CB = CS.CB, row.names = NULL)
        dat <- na.omit(dat)
        plot <- ggplot(dat, aes(CA, CB)) + theme_bw() +
                geom_point(size=1, alpha=0.5) +
                geom_density_2d() +
                geom_smooth(data = dat, method = 'lm', se = FALSE) + 
                geom_text(x = mean(dat$CA), y = mean(dat$CB), label = corr_eqn(dat$CA, dat$CB), colour = "red", size = 10) +
                labs(title = paste("Alanine", SS[i], sep=" "))
        plot <- ggMarginal(plot, type="histogram", bins=30) 
        print(plot)
        ggsave(filename = paste0("../../Output/Cysteine_Analysis/Alanine_", SS[i], ".png"), width = 4, height = 4, units = "in", plot=plot)
        assign(ala_names[i], dat)
}
save(file = paste0("../../Output/Cysteine_Analysis/Alanine_", SS[1], ".RData"), Ala_betaStrand)
save(file = paste0("../../Output/Cysteine_Analysis/Alanine_", SS[2], ".RData"), Ala_Coil)
save(file = paste0("../../Output/Cysteine_Analysis/Alanine_", SS[3], ".RData"), Ala_Helix)

Cys_names <- c("Cys_betaStrand", "Cys_Coil", "Cys_Helix")
for(i in 1:nrow(cys.filenames)){
        # print(cys.filenames[i,1])
        CS.CA <- read.table(paste0(dat.dir, cys.filenames[i,1]), quote="\"", comment.char="")
        CS.CA <- remove_outliers(unlist(CS.CA))
        CS.CB <- read.table(paste0(dat.dir, cys.filenames[i,2]), quote="\"", comment.char="")
        CS.CB <- remove_outliers(unlist(CS.CB))
        dat <- data.frame(CA = CS.CA, CB = CS.CB, row.names = NULL)
        dat <- na.omit(dat)
        plot <- ggplot(dat, aes(CA, CB)) + theme_bw() +
                geom_point(size=1, alpha=0.5) +
                geom_density_2d() +
                geom_smooth(data = dat, method = 'lm', se = FALSE) + 
                geom_text(x = mean(dat$CA), y = mean(dat$CB), label = corr_eqn(dat$CA, dat$CB), colour = "red", size = 10) +
                labs(title = paste("Cystein", SS[i], sep=" "))
        plot <- ggMarginal(plot, type="histogram", bins=30)
        print(plot)
        ggsave(filename = paste0("../../Output/Cysteine_Analysis/Cysteine_", SS[i], ".png"), width = 4, height = 4, units = "in", plot=plot)
        assign(Cys_names[i], dat)
}
save(file = paste0("../../Output/Cysteine_Analysis/Cysteine_", SS[1], ".RData"), Cys_betaStrand)
save(file = paste0("../../Output/Cysteine_Analysis/Cysteine_", SS[2], ".RData"), Cys_Coil)
save(file = paste0("../../Output/Cysteine_Analysis/Cysteine_", SS[3], ".RData"), Cys_Helix)




CysSep_names <- c("Cys_Sep_betaStrand", "Cys_Sep_Coil", "Cys_Sep_Helix")
for(i in 1:nrow(cys.filenames)){
        #print(cys.filenames[i,1])
        CS.CA <- read.table(paste0(dat.dir, cys.filenames[i,1]), quote="\"", comment.char="")
        CS.CA <- remove_outliers(unlist(CS.CA))
        CS.CB <- read.table(paste0(dat.dir, cys.filenames[i,2]), quote="\"", comment.char="")
        CS.CB <- remove_outliers(unlist(CS.CB))
        dat <- data.frame(CA = CS.CA, CB = CS.CB, row.names = NULL)
        dat <- na.omit(dat)
        clus <- clara(dat, 2)
        dat$Correlation <- as.factor(clus$clustering)
        dat1 <- dat[dat$Correlation==1,]
        dat2 <- dat[dat$Correlation==2,]
        dat$Correlation <- ifelse(dat$Correlation==1, corr_eqn(dat1$CA, dat1$CB), corr_eqn(dat2$CA, dat2$CB))

        plot <- ggplot(dat, aes(x = CA, y = CB, col = Correlation, fill = Correlation)) + theme_bw() +
                geom_point(size=1, alpha=0.5) +
                geom_density_2d() +
                geom_smooth(data = dat, method = 'lm', se = FALSE) + 
                #theme(legend.position = "bottom") +
                theme(legend.position="none") +
                labs(title = paste("Cystein", SS[i], sep=" "))
        #scale_fill_manual(name="Correlation\nValue", breaks=c(1, 2), values = c("red", "blue"), labels=c(label2, label2))
        plot <- ggMarginal(plot, type="histogram", bins=30)
        print(plot)
        ggsave(filename = paste0("../../Output/Cysteine_Analysis/Cysteine_Seperated_", SS[i], ".png"), width = 4, height = 4, units = "in", plot=plot)
        assign(CysSep_names[i], dat)
}
save(file = paste0("../../Output/Cysteine_Analysis/Cysteine_Seperated_", SS[1], ".RData"), Cys_Sep_betaStrand)
save(file = paste0("../../Output/Cysteine_Analysis/Cysteine_Seperated_", SS[2], ".RData"), Cys_Sep_Coil)
save(file = paste0("../../Output/Cysteine_Analysis/Cysteine_Seperated_", SS[3], ".RData"), Cys_Sep_Helix)
