##%######################################################%##
#                                                          #
####    Data FIltration and Covariance Recalculation    ####
#                                                          #
##%######################################################%##

# Loading data
refDB_RMSD = read.csv("../data/refDB_RMSD.csv")
refDB_RMSD = refDB_RMSD[,-1]

# Generate figures
png(file="../../output/DataFiltration_CovRecalculation/Figure1.png")
hist(refDB_RMSD$CA, breaks=100, xlim = range(0,2), xlab = "CA", main = "RMSD Histgram for CA", probability = T)
dev.off()

png(file="../../output/DataFiltration_CovRecalculation/Figure2.png")
figure2 <- hist(refDB_RMSD$CB, breaks=300, xlim = range(0,2), xlab = "CB", main = "RMSD Histgram for CB", probability = T)
dev.off()

png(file="../../output/DataFiltration_CovRecalculation/Figure3.png")
figure3 <- hist(refDB_RMSD$diff, breaks=300, xlim = range(-1,1), xlab = "CA-CB", main = "RMSD Histgram for CA-CB", probability = T)
dev.off()

png(file="../../output/DataFiltration_CovRecalculation/Figure4.png")
figure4 <- hist(abs(refDB_RMSD$diff), breaks=300, xlim = range(0,1), xlab = "CA-CB", main = "RMSD Histgram for abs(CA-CB)", probability = T)
dev.off()

png(file="../../output/DataFiltration_CovRecalculation/Figure5.png")
figure5 <- hist((refDB_RMSD$diff/(refDB_RMSD$CA+refDB_RMSD$CB)), breaks=300, xlim = range(-0.5,0.5), xlab = "CA-CB", main = "RMSD Histgram for (CA-CB)/(CA+CB)", probability = T)
dev.off()

png(file="../../output/DataFiltration_CovRecalculation/Figure6.png")
figure6 <- hist(abs(refDB_RMSD$diff/(abs(refDB_RMSD$CA)+abs(refDB_RMSD$CB))), breaks=300, xlim = range(0,0.5), xlab = "CA-CB", main = "RMSD Histgram for abs[(CA-CB)/(abs(CA)+abs(CB))]", probability = T)
dev.off()


##########################################

library(RJSONIO)
json_data <- fromJSON("../data/refDB_RMSD.json")
df <- t(data.frame(json_data))
# head(json_data)
# head(df)
colnames(df) <- c('HA', 'CA', 'CB', 'CO', 'N', 'HN')
df.cacb <-  df[, c(2,3)]
for(i in 1:nrow(df.cacb)){
        df.cacb[i,] <- as.double(df.cacb[i,])
}
df.cacb <- as.table(df.cacb)
df.cacb.naomit <- na.omit(df.cacb)
#head(df.cacb.naomit)
diff <- as.numeric(df.cacb.naomit[,1]) - as.numeric(df.cacb.naomit[,2])
df.cacb <- cbind(as.numeric(df.cacb.naomit[,1]), as.numeric(df.cacb.naomit[,2]), diff)
row.names(df.cacb) <- row.names(df.cacb.naomit)

new <- (abs(df.cacb[,3])/(abs(df.cacb[,1])+abs(df.cacb[,2])))^(1/3)
new1 <- na.omit(new)

png(file="../../output/DataFiltration_CovRecalculation/Figure7.png")
hist(new1, breaks=100, xlim = range(0,1), xlab = expression((frac(abs(CA-CB),abs(CA)+abs(CB))) ^ frac(1,3)), main = expression(paste("RMSD Histgram for ", (frac(abs(CA-CB),abs(CA)+abs(CB))) ^ frac(1,3)) ), probability = T)
dev.off()

refDB.names <- names(new)
library(mixtools)
library(ggplot2)
set.seed(111)

useable <- new1[new1<0.37]
file.name <- names(useable)
save(file.name, file = "../../output/DataFiltration_CovRecalculation/Filtered_Datasets.RData")
print(paste("Sample size will be", length(useable), sep = " "))
#file.name
nonusable <-new1[new1>0.48]
nfile.name <- names(nonusable)


file.id <- sapply(file.name, function(x){
        name <- gsub(".str", "", x)
        return(as.numeric(gsub("bmr", "", name)))
})

nonfile.id <- sapply(nfile.name, function(x){
        name <- gsub(".str", "", x)
        return(as.numeric(gsub("bmr", "", name)))
})

names(file.id) <- NULL
names(nonfile.id) <- NULL

#exportJson <- toJSON(file.id)

## Save the JSON to file
#write(exportJson, file="id.JSON")
#fromJSON("id.JSON")


infile <- file('../data/RefDB-C.db.txt', 'r')
txt <- readLines(infile)
eof <- length(txt)


i = 1
begin = c()
names = c()
while (i <= length(txt)){
        if(grepl("#DBBANK  ID: ", txt[i]) ){
                #print(txt[i])
                names <- c(names, gsub("#DBBANK  ID: ","", txt[i]))
                
        }
        if(grepl(">", txt[i])){
                begin = c(begin, i)
        }
        i = i+1
}
#length(names)==length(begin)


comm.id <- intersect(names, as.character(file.id))
non.common.id <- intersect(names, as.character(nonfile.id))
# add the end of file to the begins:
begin = c(begin, eof+1)

dat <- list()
for(i in 1:length(names)){
        start <- begin[i]+9
        end <- begin[i+1]-2
        rawtext <- data.frame(gsub("****", "NA", txt[start:end], fixed = TRUE), stringsAsFactors = F)
        
        s <- data.frame(sapply(rawtext, function(x) strsplit(sub("^\\s+", "", as.character(x)), split = "\\s+")))
        dat[names[i]] <- s
}

v <- c("A-B","A-C","A-H",
       "R-B","R-C","R-H",
       "N-B","N-C","N-H",
       "D-B","D-C","D-H",
       "C-B","C-C","C-H",
       "Q-B","Q-C","Q-H",
       "E-B","E-C","E-H", 
       "H-B","H-C","H-H",
       "I-B","I-C","I-H",
       "L-B","L-C","L-H",
       "K-B","K-C","K-H",
       "M-B","M-C","M-H",
       "F-B","F-C","F-H",
       "P-B","P-C","P-H",
       "S-B","S-C","S-H",
       "T-B","T-C","T-H",
       "Y-B","Y-C","Y-H",
       "W-B","W-C","W-H",
       "V-B","V-C","V-H")



cov.analysis <- function(id){
        covs <- list()
        for(i in v){
                aa <- strsplit(i, split = "-")[[1]][1]
                ss <- strsplit(i, split = "-")[[1]][2]
                CA <- c()
                CB <- c()
                for(j in id){
                        d <- dat[j][[1]]
                        for(k in 1:length(d)){
                                if(d[[k]][[2]]==aa & d[[k]][[3]]==ss){
                                        CA <- c(CA, as.numeric(d[[k]][[5]]))
                                        CB <- c(CB, as.numeric(d[[k]][[6]]))
                                }
                        }
                } 
                CACB <- cbind(CA, CB)
                n <- nrow(CACB)
                c <- cov(as.matrix(CACB),use = "complete")[1,2]
                r <- cor(as.matrix(CACB),use = "complete")[1,2]
                covs[[i]]<- c(n, c, r)
                #print(covs[[i]])
        }
        return(covs)
}

cov.new <- cov.analysis(comm.id)
#cov.new
# names.old <- names(dat)
# cov.old <- cov.analysis(names.old)
#cov.old

cov.old <- cov.analysis(non.common.id)

fisher.z<- function (r1,r2,n1,n2) ((0.5*log((1+r1)/(1-r1)))-(0.5*log((1+r2)/(1-r2))))/((1/(n1-3))+(1/(n2-3)))^0.5

p.value <- c()
for (i in v){
        r1 <- cov.new[[i]][3]
        r2 <- cov.old[[i]][3]
        n1 <- cov.new[[i]][1]
        n2 <- cov.old[[i]][1]
        p.value <- c(p.value, 2*(1-pnorm(abs(fisher.z(r1 ,r2,n1 ,n2 )))) )
}
p <- cbind(v, p.value)
print(cov.new)
save(file = "../../output/DataFiltration_CovRecalculation/Filtered_Cov.RData", cov.new)


