
setwd("source_code/OverlapMatrix_Weights/")
dir <- "../data/Generated-NA/"

aa = c(       "A",   "R",   "N",   "D",   "C",   "Q",   "E",   "H",   "I",   "L",   "K",   "M",   "F",   "P",   "S",   "T",   "Y",  "W", "V")
ss = c("B", "C", "H")

pop.table <- data.frame()
for(a in aa){
        HCA <- read.table(paste0(dir, a, "-H-CA.txt"))
        HCB <- read.table(paste0(dir, a, "-H-CB.txt"))
        BCA <- read.table(paste0(dir, a, "-B-CA.txt"))
        BCB <- read.table(paste0(dir, a, "-B-CB.txt"))
        CCA <- read.table(paste0(dir, a, "-C-CA.txt"))
        CCB <- read.table(paste0(dir, a, "-C-CB.txt"))
        
        HCA <- cbind(a, "H", "CA", HCA)
        colnames(HCA) <- c("Residue", "SS", "Carbon", "Value")
        HCB <- cbind(a, "H", "CB", HCB)
        colnames(HCB) <- c("Residue", "SS", "Carbon", "Value")
        BCA <- cbind(a, "B", "CA", BCA)
        colnames(BCA) <- c("Residue", "SS", "Carbon", "Value")
        BCB <- cbind(a, "B", "CB", BCB)
        colnames(BCB) <- c("Residue", "SS", "Carbon", "Value")
        CCA <- cbind(a, "C", "CA", CCA)
        colnames(CCA) <- c("Residue", "SS", "Carbon", "Value")
        CCB <- cbind(a, "C", "CB", CCB)
        colnames(CCB) <- c("Residue", "SS", "Carbon", "Value")
        
        t <- rbind.data.frame(HCA, HCB, BCA, BCB, CCA, CCB, stringsAsFactors = F)
        pop.table <- rbind.data.frame(pop.table, t)
}

requiredPackages = c("data.table", "seqinr", "plyr", "dplyr")
for(p in requiredPackages){
        if(!require(p,character.only = TRUE)) install.packages(p)
        library(p,character.only = TRUE)
}
options(digits=16)


dir <- "../data/"

AaCodesSingle.v = c("A", "B", "C", "D", "E", "F", "H", "I", "K", "L", "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y")
AaCodesSingle19.v = c("A", "C", "D", "E", "F", "H", "I", "K", "L", "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y")
AaCodes3LetterAllCap.v = c("ALA", "CYS", "ASP", "GLU", "PHE", "HIS", "ILE", "LYS", "LEU", "MET", "ASN", "PRO", "GLN", "ARG", "SER", "THR", "VAL", "TRP","TYR")
AaCodes3Letter1stCap.v = c("Ala", "Cys", "Asp", "Glu", "Phe", "His", "Ile", "Lys", "Leu", "Met", "Asn", "Pro", "Gln", "Arg", "Ser", "Thr", "Val", "Trp","Tyr")
# Reading in all the statisitcs
RefDB.StatCA <- read.csv(paste(dir, "RefDB.caStat.csv", sep=""))



# Find all the mean and sd:
# Reading in all the statisitcs
RefDB.StatCA <- read.csv(paste(dir, "RefDB.caStat.csv", sep=""))
RefDB.StatCB <- read.csv(paste(dir, "RefDB.cbStat.csv", sep=""))
RefDB.StatCB <- RefDB.StatCB[which(RefDB.StatCB$Residue!="Gly"),] # Remove the glycine
CarbonCov.t <- read.csv(paste(dir, "CarbonCovTable_BC.csv", sep=""))


# Reading in the population statistics
PopStats <- read.table(paste(dir, "aaPopStat.csv", sep=""))

# Generate average covariance
AvgCov.t <- read.csv(paste(dir, "avgCov_BC.csv", sep="")) # rowName is from the AaCodesSingle.v
# Generate covariance based on secondary structure
CarbonCov.t <- CarbonCov.t[-grep("-A", CarbonCov.t$"AA.SS"),]

# Find all the mean and sd:
CASdTable = data.frame(RefDB.StatCA$Residue, RefDB.StatCA$Coil.sd, RefDB.StatCA$Helix.sd, RefDB.StatCA$Beta.sd, RefDB.StatCA$Avg.sd)
colnames(CASdTable) = c("Residue", "C", "H", "B", "A")

CBSdTable = data.frame(RefDB.StatCB$Residue, RefDB.StatCB$Coil.sd, RefDB.StatCB$Helix.sd, RefDB.StatCB$Beta.sd, RefDB.StatCB$Avg.sd)
colnames(CBSdTable) = c("Residue", "C", "H", "B", "A")

CAMuTable = data.frame(RefDB.StatCA$Residue, RefDB.StatCA$Coil.mu, RefDB.StatCA$Helix.mu, RefDB.StatCA$Beta.mu, RefDB.StatCA$Avg.mu)
colnames(CAMuTable) = c("Residue", "C", "H", "B", "A")

CBMuTable = data.frame(RefDB.StatCB$Residue, RefDB.StatCB$Coil.mu, RefDB.StatCB$Helix.mu, RefDB.StatCB$Beta.mu, RefDB.StatCB$Avg.mu)
colnames(CBMuTable) = c("Residue", "C", "H", "B", "A")

covMatrix.E.f <- function( name ){
        
        AA.name <- unlist(strsplit(name, "-"))[1]
        SS.name <- unlist(strsplit(name, "-"))[2]
        
        sd.ca <- CASdTable[CASdTable$Residue==AA.name,][SS.name]
        sd.cb <- CBSdTable[CBSdTable$Residue==AA.name,][SS.name]
        sd.ca.double <- as.double(sd.ca)
        sd.cb.double <- as.double(sd.cb)
        
        v <- CarbonCov.t[CarbonCov.t[,1]==name,][,2]
        covMatrix <- matrix(c(sd.ca.double^2, v, v, sd.cb.double^2), 2 , 2, byrow = T)
        return(covMatrix)
}

invCovMatList.l.E = list()
for(aass in CarbonCov.t$AA.SS ){
        CovMatrix = covMatrix.E.f(aass)
        invCovMatrix = solve(CovMatrix)
        invCovMatList.l.E[[aass]] = invCovMatrix
}

caBMu <- CAMuTable[CAMuTable$Residue=="G", "B"]
caCMu <- CAMuTable[CAMuTable$Residue=="G", "C"]
caHMu <- CAMuTable[CAMuTable$Residue=="G", "H"]

caBSd <- CASdTable[CASdTable$Residue=="G", "B"]
caCSd <- CASdTable[CASdTable$Residue=="G", "C"]
caHSd <- CASdTable[CASdTable$Residue=="G", "H"]

ChiStr.f <- function(dataCaCb.v, secondaryStructure, invCovMatList) {
        names <- c()
        chiStr.v <- c()
        ss <- secondaryStructure
        dataCaCb.m <- as.matrix(dataCaCb.v)
        
        for (aa in AaCodesSingle.v) {
                name <- paste(aa, secondaryStructure, sep = "-")
                names <- c(names, name)
                caMu <- CAMuTable[CAMuTable$Residue==aa, ss]
                cbMu <- CBMuTable[CBMuTable$Residue==aa, ss]
                mean.m <- as.matrix(c(caMu, cbMu))
                invCov <- invCovMatList[[name]]
                
                dataCaCb.m <- as.matrix(dataCaCb.v)
                chiStr <- t(dataCaCb.m - mean.m) %*% invCov %*% (dataCaCb.m - mean.m)
                chiStr.v <- c(chiStr.v, chiStr)
        }
        
        names(chiStr.v) <- names
        return(chiStr.v)
}

load("../data/RefDB_prior.RData")
AA_SS_label <- prior[[987]][,1:2]



################# Glycine needed data
library(tidyverse)
load("../data/raw.RData")
length(dat)
load("../data/index.RMSD.good.RData")
length(index.RMSD.good)

dat.rmsd <- dat
cs.dat <- do.call(rbind, dat.rmsd)

cs.df <- data.frame(AA = cs.dat[,2], SS=cs.dat[,3], CA=as.numeric(cs.dat[,4]), CB=as.numeric(cs.dat[,5]), stringsAsFactors = F)
aa = "G"



glypop.table <- data.frame()
HCA <- cs.df %>% filter(AA=="G", SS=="H") %>% select(CA)
#HCB <- read.table(paste0(dir, a, "-H-CB.txt"))
BCA <- cs.df %>% filter(AA=="G", SS=="B") %>% select(CA)
#BCB <- read.table(paste0(dir, a, "-B-CB.txt"))
CCA <- cs.df %>% filter(AA=="G", SS=="C") %>% select(CA)
#CCB <- read.table(paste0(dir, a, "-C-CB.txt"))

HCA <- cbind(aa, "H", "CA", HCA)
colnames(HCA) <- c("Residue", "SS", "Carbon", "Value")
#HCB <- cbind(a, "H", "CB", HCB)
#colnames(HCB) <- c("Residue", "SS", "Carbon", "Value")
BCA <- cbind(aa, "B", "CA", BCA)
colnames(BCA) <- c("Residue", "SS", "Carbon", "Value")
#BCB <- cbind(a, "B", "CB", BCB)
#colnames(BCB) <- c("Residue", "SS", "Carbon", "Value")
CCA <- cbind(aa, "C", "CA", CCA)
colnames(CCA) <- c("Residue", "SS", "Carbon", "Value")
#CCB <- cbind(a, "C", "CB", CCB)
#colnames(CCB) <- c("Residue", "SS", "Carbon", "Value")

glypop.table <- rbind(HCA, BCA, CCA, stringsAsFactors = F)


############################################# Probability function
AaProb.f = function(chiStr){
        prob = dchisq(chiStr, df=2)
        return(prob)
}

glyProb.f = function(para){
        prob = 1 - pchisq(para^2, df=1)
        return(prob)
}



############################################# Calculation for 57 amino acids
OverlappingMatrix <- data.frame()

for (i in 1:57) {
        aa = AA_SS_label[i,1]
        ss = AA_SS_label[i,2]
        
        cs <- pop.table %>% filter(Residue==aa & SS == ss)
        cacb <- data.frame(CA = subset(cs, Carbon == "CA")["Value"], 
                           CB = subset(cs, Carbon == "CB")["Value"])
        colnames(cacb) <- c("CA", "CB")
        cacb = na.omit(cacb)
        rowProbs = c()
        for (s in c("B", "C", "H")) {
                probilities <- apply(cacb, 1, function(CaCbCS.v){AaProb.f(ChiStr.f(CaCbCS.v, secondaryStructure = s, invCovMatList.l.E))} )
                rowProbs_ss <- apply(probilities, 1, mean)
                rowProbs_ss[3] <- rowProbs_ss[2]+rowProbs_ss[3]
                rowProbs_ss <- rowProbs_ss[-2]
                rowProbs = c(rowProbs, rowProbs_ss)
        }
        
        OverlappingMatrix <- rbind(OverlappingMatrix, rowProbs)
}


colnames(OverlappingMatrix) <- paste0(AA_SS_label$AA, AA_SS_label$SS)
rownames(OverlappingMatrix) <- paste0(AA_SS_label$AA, AA_SS_label$SS)
OverlappingMatrix_norm <- scale(OverlappingMatrix, center=FALSE, scale=colSums(OverlappingMatrix))


############################################# Calculation for glycine

GlyProportionalMatrix <- data.frame()

stdNorm.f = function(value, mu, sd){
        return((value-mu)/sd)
}

mu_gly = CAMuTable %>% filter(Residue=="G")
sd_gly = CASdTable %>% filter(Residue=="G")

for (ss in c("B", "C", "H")) {

        cs <- glypop.table %>% filter(SS == ss) %>% select(Value)
        cs = as.numeric(cs$Value)
        cs = na.omit(cs)

        rowProbs = c()
        for (s in c("B", "C", "H")) {
                mean_ca <- mu_gly[s]
                sd_ca <- sd_gly[s]
                
                cs_std = unlist(sapply(cs, function(value){stdNorm.f(value, mean_ca, sd_ca)}))
                probilities <- sapply(cs_std, function(cacs){glyProb.f(cacs)} )
                rowProbs_ss <- mean(probilities)
                rowProbs = c(rowProbs, rowProbs_ss)
        }
                
        GlyProportionalMatrix <- rbind(GlyProportionalMatrix, rowProbs)
}


# Raw matrix
colnames(GlyProportionalMatrix) <- c("B", "C", "H")
rownames(GlyProportionalMatrix) <- c("B", "C", "H")

GlyProportionalMatrix_norm <- scale(GlyProportionalMatrix, center=FALSE, scale=colSums(GlyProportionalMatrix))



############################################# Collecting data
GlyOLWeights <- diag(GlyProportionalMatrix_norm)
GlyOLMatrix <- GlyProportionalMatrix_norm
AA57OLWeights <- diag(OverlappingMatrix_norm)
AA57OLMatrix <- OverlappingMatrix_norm

save(file="../../output/OverlapMatrix_Weights/Overlap_Weights.RData", GlyOLWeights, GlyOLMatrix, AA57OLWeights, AA57OLMatrix)
