##%######################################################%##
#                                                          #
####                 Robust Testing 50%                 ####
#                                                          #
##%######################################################%##
setwd("./source_code/Robust_Testing/")
dir <- "../data/"
# Import data from the raw.RData which is gennerated from the RefDB.carbon txt file, so no need to read every refDB files.
requiredPackages = c("data.table", "seqinr", "plyr")
for(p in requiredPackages){
        if(!require(p,character.only = TRUE)) install.packages(p)
        library(p,character.only = TRUE)
}

load("../data/RefDB_prior.RData")
load("../data/raw.RData")
carbonDat <- lapply(dat, function(i){
        table <- data.table(ID=i[,1], AA=i[,2], SS=i[,3], CA=as.numeric(i[,4]), CB=as.numeric(i[,5]))
        return(table)
})

carbonDat_narm <- lapply(carbonDat, function(i){
        table <- na.omit(i)
        return(table)
})

carbonDat_GU <- lapply(carbonDat, function(i){
        table <- i[which(i$AA=="G" & i$SS!="U"),]
        table$CB <- NULL
        table <- na.omit(table)
        return(table)
})

carbonDat_rmGU <- lapply(carbonDat_narm, function(i){
        table <- i[which(i$AA!="G" & i$SS!="U"),]
        return(table)
})

AA_SS_label <- prior[[987]][,1:2]
AA_SS_label <- rbind(AA_SS_label, data.frame(AA="G", SS="B"), data.frame(AA="G", SS="C"), data.frame(AA="G", SS="H"))
AA_SS_label_counts <- cbind(AA_SS_label, counts = rep(0, 60))

# prior_freq <- lapply(carbonDat, function(i){
#         actualCounts <- as.data.frame(table(i[,2:3]))
#         actualFreqs <- AA_SS_label_counts
#         for(j in 1:nrow(AA_SS_label_counts)){
#                 aa <- unlist(AA_SS_label_counts[j,]["AA"])
#                 ss <- unlist(AA_SS_label_counts[j,]["SS"])
#                 if (aa %in% actualCounts$AA & ss %in% actualCounts$SS){
#                         actualFreqs[j,3] <- actualCounts[(actualCounts$AA %in% aa & actualCounts$SS%in%ss), 3]
#                 }
#         }
#         actualFreqs[,3] <- actualFreqs[,3]/sum(actualFreqs[,3])
#         return(actualFreqs)
# })

# save(prior_freq, file = "./data/prior_freq.RData")
load("./data/prior_freq.RData")
SecondaryStructures.v  = c("B", "C", "H")
Resonances.v = c("CA", "CB")
options(digits=16)

length(carbonDat_GU)



AaCodesSingle.v = c("A", "B", "C", "D", "E", "F", "H", "I", "K", "L", "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y")
AaCodesSingle19.v = c("A", "C", "D", "E", "F", "H", "I", "K", "L", "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y")
AaCodes3LetterAllCap.v = c("ALA", "CYS", "ASP", "GLU", "PHE", "HIS", "ILE", "LYS", "LEU", "MET", "ASN", "PRO", "GLN", "ARG", "SER", "THR", "VAL", "TRP","TYR")
AaCodes3Letter1stCap.v = c("Ala", "Cys", "Asp", "Glu", "Phe", "His", "Ile", "Lys", "Leu", "Met", "Asn", "Pro", "Gln", "Arg", "Ser", "Thr", "Val", "Trp","Tyr")
# Reading in all the statisitcs
RefDB.StatCA <- read.csv(paste(dir, "RefDB.caStat.csv", sep=""))



# Find all the mean and sd:
# Reading in all the statisitcs
RefDB.StatCA <- read.csv(paste(dir, "RefDB.caStat.csv", sep=""))
RefDB.StatCB <- read.csv(paste(dir, "RefDB.cbStat.csv", sep=""))
RefDB.StatCB <- RefDB.StatCB[which(RefDB.StatCB$Residue!="Gly"),] # Remove the glycine
CarbonCov.t <- read.csv(paste(dir, "CarbonCovTable_BC.csv", sep=""))


# Reading in the population statistics
PopStats <- read.table(paste(dir, "aaPopStat.csv", sep=""))

# Generate average covariance
AvgCov.t <- read.csv(paste(dir, "avgCov_BC.csv", sep="")) # rowName is from the AaCodesSingle.v
# Generate covariance based on secondary structure
CarbonCov.t <- CarbonCov.t[-grep("-A", CarbonCov.t$"AA.SS"),]

# Find all the mean and sd:
CASdTable = data.frame(RefDB.StatCA$Residue, RefDB.StatCA$Coil.sd, RefDB.StatCA$Helix.sd, RefDB.StatCA$Beta.sd, RefDB.StatCA$Avg.sd)
colnames(CASdTable) = c("Residue", "C", "H", "B", "A")

CBSdTable = data.frame(RefDB.StatCB$Residue, RefDB.StatCB$Coil.sd, RefDB.StatCB$Helix.sd, RefDB.StatCB$Beta.sd, RefDB.StatCB$Avg.sd)
colnames(CBSdTable) = c("Residue", "C", "H", "B", "A")

CAMuTable = data.frame(RefDB.StatCA$Residue, RefDB.StatCA$Coil.mu, RefDB.StatCA$Helix.mu, RefDB.StatCA$Beta.mu, RefDB.StatCA$Avg.mu)
colnames(CAMuTable) = c("Residue", "C", "H", "B", "A")

CBMuTable = data.frame(RefDB.StatCB$Residue, RefDB.StatCB$Coil.mu, RefDB.StatCB$Helix.mu, RefDB.StatCB$Beta.mu, RefDB.StatCB$Avg.mu)
colnames(CBMuTable) = c("Residue", "C", "H", "B", "A")

covMatrix.E.f <- function( name ){
        
        AA.name <- unlist(strsplit(name, "-"))[1]
        SS.name <- unlist(strsplit(name, "-"))[2]
        
        sd.ca <- CASdTable[CASdTable$Residue==AA.name,][SS.name]
        sd.cb <- CBSdTable[CBSdTable$Residue==AA.name,][SS.name]
        sd.ca.double <- as.double(sd.ca)
        sd.cb.double <- as.double(sd.cb)
        
        v <- CarbonCov.t[CarbonCov.t[,1]==name,][,2]
        covMatrix <- matrix(c(sd.ca.double^2, v, v, sd.cb.double^2), 2 , 2, byrow = T)
        return(covMatrix)
}

#test.cov.B <- covMatrix.B.f(name = "B-C")
#test.cov.B

invCovMatList.l.E = list()
for(aass in CarbonCov.t$AA.SS ){
        CovMatrix = covMatrix.E.f(aass)
        invCovMatrix = solve(CovMatrix)
        invCovMatList.l.E[[aass]] = invCovMatrix
}
# Glycine Probability #######################################################################################
# glyProb.f = function(para){
#         prob = 2*pnorm(-abs(para))
#         return(prob)
# }

# caBMu <- CAMuTable[CAMuTable$Residue=="G", "B"]
# caCMu <- CAMuTable[CAMuTable$Residue=="G", "C"]
# caHMu <- CAMuTable[CAMuTable$Residue=="G", "H"]
# 
# caBSd <- CASdTable[CASdTable$Residue=="G", "B"]
# caCSd <- CASdTable[CASdTable$Residue=="G", "C"]
# caHSd <- CASdTable[CASdTable$Residue=="G", "H"]

# All the rest 19 AAs #######################################################################################
ChiStr.f <- function(dataCaCb.v, secondaryStructure, invCovMatList) {
        names <- c()
        chiStr.v <- c()
        ss <- secondaryStructure
        dataCaCb.m <- as.matrix(dataCaCb.v)
        
        for (aa in AaCodesSingle.v) {
                name <- paste(aa, secondaryStructure, sep = "-")
                names <- c(names, name)
                caMu <- CAMuTable[CAMuTable$Residue==aa, ss]
                cbMu <- CBMuTable[CBMuTable$Residue==aa, ss]
                mean.m <- as.matrix(c(caMu, cbMu))
                invCov <- invCovMatList[[name]]
                
                dataCaCb.m <- as.matrix(dataCaCb.v)
                chiStr <- t(dataCaCb.m - mean.m) %*% invCov %*% (dataCaCb.m - mean.m)
                chiStr.v <- c(chiStr.v, chiStr)
        }
        
        names(chiStr.v) <- names
        return(chiStr.v)
}

AaProb.f = function(chiStr){
        prob = dchisq(chiStr, df=2)
        return(prob)
}

SumAbsDiff.f <- function(step, dat_cacb, dat_ca, aaFreq, invCovMatList, ss_prior) {
        sumAaProb.v <- rep(0, 57)
        testDataLen <- nrow(dat_cacb)
        
        
        for (i in 1:testDataLen) {
                aaProb.v <- c()
                CaCbCS.v <- c(dat_cacb[i, 1] + step, dat_cacb[i, 2] + step)
                CaCbCS.v.flip <- rev(CaCbCS.v)
                
                # a = ChiStr.f(CaCbCS.v, secondaryStructure = "H", invCovMatList)
                # b = ChiStr.f(CaCbCS.v.flip, secondaryStructure = "H", invCovMatList)
                # 
                # c = rbind(a, b)
                # d <- apply(c,2,min)
                
                chiStrH.v <- apply(rbind(ChiStr.f(CaCbCS.v, secondaryStructure = "H", invCovMatList), 
                                         ChiStr.f(CaCbCS.v.flip, secondaryStructure = "H", invCovMatList)), 2, min)
                chiStrB.v <- apply(rbind(ChiStr.f(CaCbCS.v, secondaryStructure = "B", invCovMatList), 
                                         ChiStr.f(CaCbCS.v.flip, secondaryStructure = "B", invCovMatList)), 2, min)
                chiStrC.v <- apply(rbind(ChiStr.f(CaCbCS.v, secondaryStructure = "C", invCovMatList), 
                                         ChiStr.f(CaCbCS.v.flip, secondaryStructure = "C", invCovMatList)), 2, min)
                
                chiStr.v <- c(chiStrB.v, chiStrC.v, chiStrH.v)  # 19 AA SS Order: B, C, H
                
                Prob <- AaProb.f(chiStr.v)
                Prob[3] <- Prob[2] + Prob[3]
                Prob[23] <- Prob[22] + Prob[23]
                Prob[43] <- Prob[42] + Prob[43]
                Prob <- Prob[-c(2,22,42)]
                
                PostProb <- Prob#*ss_prior[1:57]
                sumAaProb.v <- sumAaProb.v + PostProb
        }
        
        #testDataLen_ca <- nrow(dat_ca)
        #glyProb <- rep(0, 3)
        # caCs <- dat_ca + step
        # for (i in 1:testDataLen_ca) {
        #         z <- c( (caCs[i] - caBMu)/caBSd, (caCs[i] - caHMu)/caHSd, (caCs[i] - caCMu)/caCSd ) # Order: B, C, H
        #         #print(z)
        #         glyProb <- glyProb + glyProb.f(z)
        # }
        
        ########## Priors are here
        
        #sumAaProb.v <- c(sumAaProb.v, glyProb*ss_prior[58:60])
        #sumAaProb.v <- sumAaProb.v/sum(sumAaProb.v)
        
        ##### Control added here 3, 15
        sumAaProb.v[1:57] <- sumAaProb.v[1:57]*AA57OLWeights
        sumAaProb.v <- sumAaProb.v/sum(sumAaProb.v)
        
        
        
        actualFreq.v <- aaFreq$tmpPrior[1:57]
        actualFreq.v[1:57] <- actualFreq.v[1:57] %*% as.matrix(AA57OLMatrix)
        actualFreq.v[1:57] <- actualFreq.v[1:57]*AA57OLWeights
        actualFreq.v <- actualFreq.v/sum(actualFreq.v)
        #print(length(actualFreq.v))
        
        #print(length(sumAaProb.v*control))
        sumAbsDiff <- sum(abs(sumAaProb.v*control - actualFreq.v*control))
        
        return(sumAbsDiff)
}

####################### RefCorr #######################
RefCorr.f = function(dat_ca, error, from, to, dat_cacb, invCovMatList, sequence, ss_prior, aaFreq){
        stepNum = 50
        N <- seq(from, to, length.out = stepNum)
        #print(sequence)
        sumAbsDiff.v <- vector(mode = "double", length=stepNum)
        for(i in 1:stepNum){
                step <- N[i]
                sumAbsDiff.v[i] <- SumAbsDiff.f(step, dat_cacb, dat_ca, aaFreq, invCovMatList, ss_prior)
                # print(sumAbsDiff.v)
        }
        # sumAbsDiff.t <- cbind(N, sumAbsDiff.v)
        # colnames(sumAbsDiff.t) <- c("steps", "sumAbsDiff")
        # print(sumAbsDiff.v)
        return(sumAbsDiff.v)
}

####################### Testing function #######################
testing.f = function(dat_ca, error, from, to, dat_cacb, invCovMatList,  sequence, ss_prior){
        
        sequence = c2s(sequence)
        aaFreq = ss_prior
        ss_prior$tmpPrior[ss_prior$tmpPrior!=0] <- 1
        ss_prior <- ss_prior$tmpPrior
        
        result1 = RefCorr.f(dat_ca, error, from, to, dat_cacb, invCovMatList, sequence, ss_prior, aaFreq)
        if(!is.double(result1)) return()
        N = seq(from, to, length.out = 50)
        #plot(N,result1)
        lowest1 = N[which.min(result1)]
        # print(lowest1)
        f = lowest1 - 1
        t = lowest1 + 1
        result2 = RefCorr.f(dat_ca, error, from=f, to=t, dat_cacb, invCovMatList, sequence, ss_prior, aaFreq)
        N = seq(f, t, length.out = 50)
        #plot(N,result2)
        lowest2 = N[which.min(result2)]
        #print(lowest2)
        result = list()
        result[[1]] = result1
        result[[2]] = result2
        result[[3]] = lowest1
        result[[4]] = lowest2
        return(result)
}

library(parallel)
load("../data/overLappingMatrix.RData")


load("../data/index90.RData")
AA57OLMatrix <- AA57OLMatrix
AA57OLWeights <- AA57OLWeights


carbonDat_GU <- carbonDat_GU[index90]
carbonDat_rmGU <- carbonDat_rmGU[index90]
prior <- prior[index90]


control <- rep(1, len=60)
control[58:60] <- 0
                
percent = 0.55
results_55 <- mclapply(1:length(carbonDat_GU), function(k){
        print(paste("################ Processing dataset No. ", k, " ################"))
        tmpGlyDat <- as.data.frame(carbonDat_GU[[k]])
        tmpDat <- as.data.frame(carbonDat_rmGU[[k]])
        if(nrow(tmpGlyDat) > 0 & nrow(tmpDat) > 0){
                ca <- as.matrix(tmpGlyDat[,4])
                seq <- tmpDat[,2]
                ss <- tmpDat[,3]
                
                tmpDat <- tmpDat[sample(1:nrow(tmpDat), round(nrow(tmpDat)*percent, digits = 0), replace=FALSE),]
                
                cacb <- as.matrix(tmpDat[,4:5])
                
                tmpPrior <- apply(AA_SS_label, 1, function(x){
                        aa <- x[1]
                        ss <- x[2]
                        
                        pp <- prior[[k]]$Freq[which(prior[[k]]$AA==aa & prior[[k]]$SS==ss)]
                        ssPrior <- ifelse(length(pp)==0, 0, pp)
                        return(ssPrior)
                        
                })
                tmpPrior <- cbind(AA_SS_label, tmpPrior)
                # tmpPrior <- rep(1, len=60)
                # tmpPrior <- control
                
                result <- testing.f(dat_ca = ca, error = 0, from = -5, to = 5, dat_cacb=cacb, invCovMatList = invCovMatList.l.E,  sequence=seq, ss_prior=tmpPrior)
                #print(result)
                #cat(result)
                return(result)
        }
}, mc.cores = 1)

save(results_55, file="../../output/Robust_Testing/Robust_Testing_55.RData")


